#include <stdlib.h>

#include <glib.h>

#include <vhttpd/codes.h>
#include <vhttpd/channel.h>

#include "private.h"
#include "channel_private.h"
/*** functions */

int
channel_get_fd (const channel_t * c)
{
    return c ? c->fd : -1;
}

const char *
channel_get_remoteaddr (const channel_t * c)
{
    return c ? c->remote_addr : "";
}

int
channel_get_type (const channel_t * c)
{
    return c ? c->type : CHANNEL_TYPE_UNKNOWN;
}

channel_t *
channel_accept (channel_t * c)
{
  if (c && c->accept)
    return c->accept (c);
  else
    return NULL;
}

int
channel_send (channel_t * c, const char *buf, size_t len)
{
  if (c && c->send)
    return c->send (c, buf, len);
  else
    return 1;
}

int
channel_sendfile (channel_t * c, int fd, size_t len)
{
  if (c && c->sendfile)
    return c->sendfile (c, fd, len);
  else
    return 1;
}

int
channel_receive (channel_t * c, buffer_t *buf)
{
  if (c && c->receive)
    return c->receive (c, buf);
  else
    return 1;
}

void
destroy_channel (channel_t * c)
{
  if (c && c->destroy)
    c->destroy (c);
}

channel_t *
create_channel_from_socket (int socket_fd, const char *remote_addr, int flags)
{
    channel_t *c;

    if (socket_fd < 0)
        return NULL;

    c = g_new0 (channel_t, 1);
    plain_channel_init (c, socket_fd, g_strdup (remote_addr), flags);
    return c;
}

channel_t *
create_inet_channel (const char *addr, const char *service, int flags)
{
    int s;

    if((s = create_inet_socket(addr, service, flags)) < 0)
        return NULL;

    return create_channel_from_socket (s, g_strdup (addr), flags);
}

channel_t *
create_unix_channel (const char *path, int flags)
{
    int s;

    if((s = create_unix_socket(path, flags)) < 0)
        return NULL;

    return create_channel_from_socket (s, g_strdup (path), flags);
}

void
channel_unix_set_ownership (channel_t *c, const char *user, const char *group)
{
    unix_socket_set_ownership(c->remote_addr, user, group);
}

