#include <vhttpd/codes.h>

typedef struct error_entry_t
{
  retcode_t num;
  const char *desc;
} error_entry_t;

static const error_entry_t errors[] = {
  /* 0 */
  {unfinished, "Unfinished"},	/* offset: 0 */
  /* 100 */
  {do_continue, "Do continue"},	/* offset: 1 */
  {switching_protocols, "Switching protocols"},
  /* 200 */
  {ok, "OK"},			/* offset: 3 */
  {created, "Created"},
  {accepted, "Accepted"},
  {non_authoritative_information, "Non authoritative information"},
  {no_content, "No content"},
  {reset_content, "Reset content"},
  {partial_content, "Partial content"},
  /* 300 */
  {multiple_choices, "Multiple choices"},	/* offset: 10 */
  {moved_permanently, "Moved permanently"},
  {found, "Found"},
  {see_other, "See other"},
  {not_modified, "Not modified"},
  {use_proxy, "Use proxy"},
  {unused, "Unused"},
  {temporary_redirect, "Temporary redirect"},
  /* 400 */
  {bad_request, "Bad request"},	/* offset: 18 */
  {unauthorized, "Unauthorized"},
  {payment_required, "Payment required"},
  {forbidden, "Forbidden"},
  {not_found, "Not found"},
  {method_not_allowed, "Method not allowed"},
  {not_acceptable, "Not acceptable"},
  {proxy_authentication_required, "Proxy authentication required"},
  {request_timeout, "Request timeout"},
  {conflict, "Conflict"},
  {gone, "Gone"},
  {length_required, "Length required"},
  {precondition_failed, "Precondition failed"},
  {request_entity_too_large, "Request entity too large"},
  {request_uri_too_large, "Request uri too large"},
  {unsupported_media_type, "Unsupported media type"},
  {requested_range_not_satisfiable, "Request range not satisfiable"},
  {expectation_failed, "Expectation failed"},
  /* internal server error */
  {internal_server_error, "Internal server error"},	/* offset: 36 */
  {not_implemented, "Not implemented"},
  {bad_gateway, "Bad gateway"},
  {service_unavailable, "Service unavailable"},
  {gateway_timeout, "Gateway timeout"},
  {http_version_not_supported, "HTTP version not supported"}
};

static const int max_error = sizeof (errors) / sizeof (error_entry_t);

static const int offsets[] = { 0, 1, 3, 10, 18, 36 };
static const int max_offset = 5;

static const char invalid_error[] = "Invalid error code";

const char *
http_strerror (retcode_t code)
{
  int c = code / 100;
  int sc = code % 100;
  int n;

  if (c < 0 || c > max_offset)
    return invalid_error;

  n = offsets[c] + sc;

  if (n >= max_error || errors[n].num != code)
    return invalid_error;

  return errors[n].desc;
}
