#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <grp.h>
#include <pwd.h>

#include <unistd.h>

#include <errno.h>

#include <vhttpd/log.h>
#include <vhttpd/daemon.h>

#include "private.h"

/*** helpers */

static void
drop_group (const char *name)
{
  struct group *gr;

  ASSERT_STRERROR((gr = getgrnam (name)) != NULL);
  ASSERT_STRERROR(!setgid (gr->gr_gid));
}

static void
drop_user (const char *name)
{
  struct passwd *pw;

  ASSERT_STRERROR((pw = getpwnam (name)) != NULL);
  ASSERT_STRERROR(!initgroups (pw->pw_name, pw->pw_gid));
  ASSERT_STRERROR(!setuid (pw->pw_uid));
}

static int
acquire_pid (const char *pidfile)
{
  int fd;

  ASSERT_STRERROR((fd = open (pidfile, O_RDWR | O_CREAT, 0600)) > 0);

  if (fd <= STDERR_FILENO)
    {
      ASSERT_STRERROR(dup2 (fd, STDERR_FILENO + 1) > 0);
      close (fd);
      fd = STDERR_FILENO + 1;
    }

  close_on_exec(fd);

  if (flock (fd, LOCK_EX | LOCK_NB) < 0)
    {
      fprintf (stderr, "can't lock pidfile\n");
      close (fd);
      return -1;
    }

  return fd;
}

static void
write_pid (int fd)
{
  char buf[30];
  int blen;

  blen = snprintf (buf, sizeof buf, "%ld", (long) getpid ());

  if (blen <= 0 || (size_t) blen > sizeof buf)
    RUNTIME_ERROR ("snprintf:%m");

  ASSERT_STRERROR(TEMP_FAILURE_RETRY (write (fd, buf, strlen (buf))) == blen);
  ASSERT_STRERROR(!ftruncate (fd, (off_t) blen));
  close_on_exec(fd);
  ASSERT_STRERROR(!fchmod (fd, 0644));
}


/*** functions */

void
drop_privs (const char *user, const char *group)
{
  if (!user || !*user || !group || !*group)
    RUNTIME_ERROR ("user or group parameter is missing");

  ASSERT_STRERROR(!setgroups (0, NULL));

  drop_group (group);
  drop_user (user);
}

void
daemonize (const char *pidfile)
{
  int pidfd;

  if (!pidfile || !*pidfile)
    RUNTIME_ERROR ("server-pidfile parameter is missing");

  if ((pidfd = acquire_pid (pidfile)) < 0)
    RUNTIME_ERROR ("already running");

  ASSERT_STRERROR(!daemon (0, 0));

  write_pid (pidfd);
}
