#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>

#include <glib.h>

#include <vhttpd/decode.h>

/*** helpers */

#define XX      255		/* illegal base64 char */
#define EQ      254		/* padding */

static unsigned char _index_64[256] = {
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, 62, XX, XX, XX, 63,
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, XX, XX, XX, EQ, XX, XX,
  XX, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, XX, XX, XX, XX, XX,
  XX, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, XX, XX, XX, XX, XX,

  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
  XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX, XX,
};

static void *
_zero (void *item)
{
  g_free (item);
  return NULL;
}

/*** functions */
char *
url_encode (const char *in, size_t len)
{
  char *result;
  char *out;
  size_t outlen = 0;

  if (!in)
    return NULL;

  out = result = g_malloc0 (len*3+1);

  for (; *in && len > 0; ++in, --len)
    {
      if (('a' <= *in && *in <= 'z') ||
          ('A' <= *in && *in <= 'Z') ||
          ('0' <= *in && *in <= '9'))
        {
	  *(out++)=*in;
	  outlen++;
	}
      else if (*in == ' ')
      {
        *(out++)='+';
        outlen++;
      }
      else
      {
        char buf[3];
        snprintf(buf,3,"%02x",(char)(*in));

        *(out++)='%';
        *(out++)=buf[0];
        *(out++)=buf[1];
        outlen+=3;
      }
   }

  if ((result = g_realloc(result,outlen+1)) == NULL)
    return NULL;
  else
    return result;
}


char *
url_decode (const char *in, size_t len)
{
  char *result;
  char *out;

  if (!in)
    return NULL;

  out = result = g_malloc0 (len + 1);

  for (; *in && len > 0; ++in, --len)
    {
      switch (*in)
	{
	case '%':
	  {
	    char buf[3];
	    int c;
	    char *err;

	    if (!*(++in))
	      return _zero (result);
	    buf[0] = *in;

	    if (!*(++in))
	      return _zero (result);
	    buf[1] = *in;
	    buf[2] = '\0';

	    len -= 2;

	    errno = 0;
	    c = strtoul (buf, &err, 16);
	    if (*err || errno)
	      return _zero (result);
	    *(out++) = (char) c;
	  }
	  break;
	case '+':
	  *(out++) = ' ';
	  break;
	default:
	  *(out++) = *in;
	  break;
	}
    }
  return result;
}

char *
base64_decode (const char *in, int dowarn)
{
  char *result;
  char *out;
  unsigned char c[4];

  if (!in)
    return NULL;

  out = result = g_malloc0 (strlen (in) + 1);

  while (*in)
    {
      int i = 0;
      do
	{
	  unsigned char uc = _index_64[(int) *in++];

	  if (uc != XX)
	    c[i++] = uc;
	  if (!*in)
	    {
	      if (i < 4)
		{
		  if (dowarn && i)
		    fprintf (stderr, "Premature end of base64 data\n");
		  if (i < 2)
		    return result;
		  if (i == 2)
		    c[2] = EQ;
		  c[3] = EQ;
		}
	      break;
	    }
	}
      while (i < 4);

      if (c[0] == EQ || c[1] == EQ)
	{
	  if (dowarn)
	    fprintf (stderr, "Premature padding of base64 data\n");
	  break;
	}

      *out++ = (c[0] << 2) | ((c[1] & 0x30) >> 4);
      if (c[2] == EQ)
	break;
      *out++ = ((c[1] & 0x0f) << 4) | ((c[2] & 0x3c) >> 2);
      if (c[3] == EQ)
	break;
      *out++ = ((c[2] & 0x03) << 6) | c[3];
    }
  return result;
}
