#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <string.h>

#include <glib.h>

#include <vhttpd/time.h>
#include <vhttpd/http_message.h>

/*** helpers */

static const char *_indexlist[] =
  { "index.html", "index.htm", "index.xhtml", NULL };

static const char *
_mime_type (const char *name)
{
  char *ext;

  if (!name || !(ext = strrchr (name, '.')))
    return "text/plain; charset=utf-8";
  else
    ext++;

  if (!strcmp (ext, "htm") || !strcmp (ext, "html"))
    return "text/html; charset=utf-8";
  if (!strcmp (ext, "css"))
    return "text/css";
  if (!strcmp (ext, "js"))
    return "application/javascript";
  if (!strcmp (ext, "png"))
    return "image/png";
  if (!strcmp (ext, "gif"))
    return "image/gif";
  if (!strcmp (ext, "jpg") || !strcmp (ext, "jpeg"))
    return "image/jpeg";

  return "text/plain; charset=utf-8";
}

static message_t *
_open_file (const char *path, const char *suffix, time_t mtime)
{
  char *str =
    g_strdup_printf ("%s%s%s", path, suffix ? "/" : "", suffix ? suffix : "");
  int fd = -1;
  struct stat st;
  message_t *r = NULL;

  if (((fd = open (str, O_RDONLY)) >= 0) &&
      !fstat (fd, &st) && S_ISREG (st.st_mode))
    {
      if (gmt_time (st.st_mtime) > mtime)
	{
	  char *m_time = time_string (st.st_mtime);
	  r = create_response_message (ok);
	  message_set_file (r, fd, st.st_size);
	  message_set_header (r, "last-modified", m_time);
	  message_set_header (r, "content-type", _mime_type (str));

	  g_free (m_time);
	}
      else
        {
	r = create_response_message (not_modified);
	message_set_header (r, "content-type", _mime_type (str));
	}
    }

  if (fd >= 0)
    close (fd);
  g_free (str);

  return r;
}

/*** functions */

message_t *
message_file_new (const char *path, const char *mtime)
{
  message_t *r = NULL;
  const char **i;
  time_t m = time_parse (mtime);

  if (!path)
    return NULL;

  if ((r = _open_file (path, NULL, m)) != NULL)
    return r;

  for (i = _indexlist; *i; ++i)
    {
      if ((r = _open_file (path, *i, m)) != NULL)
	return r;
    }

  return NULL;
}

message_t *
message_redirect_new_printf (const char *fmt, ...)
{
  va_list ap;

  message_t *r = create_response_message (found);

  va_start (ap, fmt);
  message_set_header_vprintf (r, "location", fmt, ap);
  va_end (ap);
  return r;
}

message_t *
message_redirect_new (const char *location)
{
  message_t *r = create_response_message (found);
  message_set_header (r, "location", location);
  return r;
}
