#define _GNU_SOURCE
#include <stdlib.h>
#include <time.h>
#include <locale.h>
#include <ctype.h>

#include <string.h>

#include <glib.h>

#include <errno.h>

#include <vhttpd/time.h>
#include <vhttpd/log.h>

/*** helpers */
static const char *_day_of_week[] =
  { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static const char *_month[] =
  { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct",
  "Nov", "Dec"
};

/*** functions */

char *
time_string (const time_t t)
{
  struct tm now_buf;
  struct tm *now;

  if ((now = gmtime_r (&t, &now_buf)) == NULL)
    runtime_error ("time_string:gmtime_r:%m");

  return g_strdup_printf ("%s, %02d %s %04d %02d:%02d:%02d GMT",
			  _day_of_week[now->tm_wday],
			  now->tm_mday,
			  _month[now->tm_mon],
			  now->tm_year + 1900,
			  now->tm_hour, now->tm_min, now->tm_sec);
}

time_t
time_parse (const char *str)
{
  struct tm t;
  time_t result = 0;
  const char *l;

  if (!str)
    return 0;

  while (isblank (*str))
    ++str;

  memset (&t, 0, sizeof (t));

  l = setlocale (LC_TIME, "C");
  if (strptime (str, "%a, %d %b %Y %T GMT", &t) /* RFC1123 */  ||
      strptime (str, "%A, %d-%b-%y %T GMT", &t) /* RFC850 */  ||
      strptime (str, "%a %b %e %T %Y", &t))	/* asctime(3) */
    result = mktime (&t);
  setlocale (LC_TIME, l);

  return (result < 0) ? 0 : result;
}

time_t
gmt_time (time_t t)
{
  time_t gt;
  struct tm ts;

  if (!t)
    return 0;

  if (!gmtime_r (&t, &ts))
    runtime_error ("gmt_time:gmtime_r:%m");
  if ((gt = mktime (&ts)) == -1)
    runtime_error ("gmt_time:mktime:%m");

  return gt;
}
